"""
CommCare/Export-specific extensions to MiniLinq. 

To date, this is simply built-ins for querying the
API directly.
"""

import simplejson
from commcare_export.env import DictEnv, CannotBind, CannotReplace


class CommCareHqEnv(DictEnv):
    """
    An environment providing primitives for pulling from the
    CommCareHq API.
    """
    
    def __init__(self, commcare_hq_client, since=None):
        self.commcare_hq_client = commcare_hq_client
        self.since = since
        super(CommCareHqEnv, self).__init__({
            'api_data' : self.api_data
        })

    def api_data(self, resource, payload=None):
        payload = dict(payload or {}) # Do not mutate passed-in dicts
        params = {'limit': 100}

        # Currently the form resource endpoint and the case resource endpoint are completely different
        if resource == 'form':
            if self.since:
                if 'filter' not in payload: payload['filter'] = {}

                payload['filter'] = {
                    "and": [
                        payload.get("filter", {"match_all":{}}),
                        {'range': {'received_on': {'from': self.since.isoformat()}}},
                    ]
                }

            if payload:
                params.update({'_search': simplejson.dumps(payload, separators=(',',':'))}) # compact encoding
                
        elif resource == 'case':
            if self.since:
                payload['server_date_modified_start'] = self.since.isoformat()

            if payload:
                params.update(payload)

        else:
            raise ValueError('I do not know how to access the API resource "%s"' % resource)
        
        return self.commcare_hq_client.iterate(resource, params=params)

    def bind(self, name, value):
        raise CannotBind()

    def replace(self, data):
        raise CannotReplace()



import os.path
import sys
import glob
import io
import subprocess
import setuptools 
from setuptools.command.test import test as TestCommand

# Build README.txt from README.md if not present, and if we are actually building for distribution to pypi
if not os.path.exists('README.txt') and 'sdist' in sys.argv:
    subprocess.call(['pandoc', '--to=rst', '--output=README.txt', 'README.md'])

readme = 'README.txt' if os.path.exists('README.txt') else 'README.md'

class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(['tests/'] + self.test_args)
        sys.exit(errno)

setuptools.setup(   
    name = "commcare-export",
    version = "0.10.5",
    description = 'A command-line tool (and Python library) to extract data from CommCareHQ into a SQL database or Excel workbook',
    long_description = io.open(readme, encoding='utf-8').read(),
    author = 'Dimagi',
    author_email = 'information@dimagi.com',
    url = "https://github.com/dimagi/commcare-export",
    entry_points = { 'console_scripts': ['commcare-export = commcare_export.cli:entry_point'] },
    packages = ['commcare_export'],
    data_files = [(os.path.join('share', 'commcare-export', 'examples'), glob.glob('examples/*.json') + glob.glob('examples/*.xlsx'))],
    license = 'MIT',
    install_requires = ['jsonpath-rw>=1.1',
                        'openpyxl',
                        'six',
                        'openpyxl',
                        'requests',
                        'simplejson',
                        'python-dateutil',
                        'sqlalchemy',
                        'alembic',
                        'argparse'],
    tests_require = ['pytest', 'psycopg2'],
    cmdclass = {'test': PyTest},
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Healthcare Industry',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Database',
        'Topic :: Software Development :: Interpreters',
        'Topic :: System :: Archiving',
        'Topic :: System :: Distributed Computing',
    ],
)




class RepeatableIterator(object):
    """
    Pass something iterable into this and, 
    unless it has crufty issues, voila.
    """
    
    def __init__(self, generator):
        self.generator = generator

    def __iter__(self):
        return self.generator()

    @classmethod
    def to_jvalue(cls, obj):
        if isinstance(obj, cls):
            return list(obj)
        raise TypeError(repr(o) + 'is not JSON serializable')



from __future__ import unicode_literals, print_function, absolute_import, division, generators, nested_scopes
import hashlib
import io
import json

def digest_file(path):
    with io.open(path, 'rb') as filehandle:
        digest = hashlib.md5()
        while True:
            chunk = filehandle.read(4096) # Arbitrary choice of size to be ~filesystem block size friendly
            if not chunk:
                break
            digest.update(chunk)
        return digest.hexdigest()
